function [feats] = FOVQA(frame1, frame2, block_size)
%FOVQA is a blind foveated video quality feature extractor. It is the
%implementation of the paper:
%   Y. Jin, A. Patney, A. C. Bovik, "FOVQA: Blind Foveated Video Quality
%   Assessment", submitted to IEEE Transactions on Image Processing.
%   frame1 and frame2: adjacent frames as input.
%   block_size: block size for space-variant estimations. Usually set to 32
%   or larger.
tic;
Ws = 0.01; Cs = 0.1;
factor = min(size(frame1))/block_size/32;
sigma_Gs = (0.5:0.25:2)*factor;
weight_patterns = genWeightPatterns(floor(size(frame1)/block_size),10,20,factor);
weight_patterns_1 = genWeightPatterns(floor(size(frame1)/block_size/2),10,20,0.5*factor);
weight_patterns_falloff = genWeightPatterns(floor(size(frame1)/block_size),6,13,factor);
[feat_map_18,feat_map_9] = genFeatsMaps(frame1,block_size,Ws,Cs);
feats = radialBasisExtraction(feat_map_18,weight_patterns);
feats = [feats,radialBasisExtraction(feat_map_9,weight_patterns_1)];

QP_pred_feats = feat_map_18(:,:,[1,2]);
QP_pred_feats(:,:,3) = imresize(feat_map_9(:,:,1),2);
QP_pred_feats(:,:,2:3) = sqrt(QP_pred_feats(:,:,2:3));
QP_map = predictQP(QP_pred_feats);
QP_gradient = computeGradient(QP_map, sigma_Gs);
feats = [feats, radialBasisExtraction(QP_gradient,weight_patterns_falloff)];

Wt = 0.01; Ct = 0.1;
shifts = [1,0;0,1;-1,0;0,-1];
frame_diff = zeros(size(frame1,1),size(frame1,2),size(shifts,1));
for itr_shift = 1:size(shifts,1)
    frame_diff(:,:,itr_shift) = frame2-circshift(frame1,shifts(itr_shift,:));
end
[feat_map_temporal_18,feat_map_temporal_9] = genFeatsMaps(frame_diff,block_size,Wt,Ct);
feats = [feats, radialBasisExtraction(feat_map_temporal_18,weight_patterns)];
feats = [feats, radialBasisExtraction(feat_map_temporal_9,weight_patterns_1)];
toc;
end
